﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;

namespace GridTest2
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : INotifyPropertyChanged
	{
		public static MainWindow Instance { get; set; }

		public ICommand RemoveItemCommand { get; set; }

		public MainWindow()
		{
			InitializeComponent();
			DataContext = this;
			Instance = this;

			Items = new ObservableCollection<AddressBookRecord>();
			
			applyXamGridHacks();

			xamGrid.CellEnteredEditMode += (s, e) =>
			{
				if (EditedRow == null)
				{
					EditedRow = e.Cell.Row.Data as AddressBookRecord;
					if (!IsAdding)
						IsEditing = true;
				}
			};

			xamGrid.RowExitedEditMode += (s, e) =>
			{
				Debug.WriteLine("Row exited edit mode!");
				IsEditing = false;
				EditedRow = null;
				if (IsAdding)
					hideNewRow();
				Debug.WriteLine("Exit completed!");
			};

			xamGrid.RowExitingEditMode += (s, e) =>
			{
				var data = e.Row.Data as AddressBookRecord;
				Debug.WriteLine(string.Format("Exiting: data is {0}", data.IsOkay() ? "valid" : "invalid"));
				if (!e.EditingCanceled && !data.IsOkay())
					e.Cancel = true;
			};

			RemoveItemCommand = new RoutedCommand("RemoveItemCommand", GetType());
			CommandBindings.Add(new CommandBinding(RemoveItemCommand, removeItemCommand_Executed));

			SelectedItems = new ListCollectionView(Items);
			SelectedItems.Filter = new Predicate<object>(
				obj =>
				{
					var rec = obj as AddressBookRecord;
					return rec != null && rec.RelatesToQuery(SearchQuery);
				}
			);
		}

		private void applyXamGridHacks()
		{
			// HACK #1
			// Disable mouse capture to prevent flickering
			// Mouse capturing is required when scrolling (OriginalSource = Thumb) and column resizing (OriginalSource = HeaderCellControl).
			Mouse.AddGotMouseCaptureHandler(xamGrid,
				(sender, args) =>
				{
					if (args.OriginalSource is XamGrid)
						Mouse.Capture(null);
				}
			);

			// HACK #2
			// The "new row" editors cannot be activated when binding over an empty ListCollectionView.
			// So we create an item and remove it through the dispatcher.
			Loaded += (s, e) =>
			{
				if (Items.Count() == 0)
				{
					Items.Add(new AddressBookRecord());
					Dispatcher.BeginInvoke(
						new Action(() => Items.RemoveAt(0))
					);
				}
			};

			// HACK #3
			// Prevent xamGrid from stealing focus when the source ListCollectionView is refreshed.
			xamGrid.ActiveCellChanging += (s, e) =>
			{
				if (!Keyboard.IsKeyDown(Key.Tab))
					e.Cancel = true;
			};

			// HACK #4
			// Maintain the ListCollectionView state when the user has cancelled edit mode.
			xamGrid.CellExitingEditMode += (s, e) =>
			{
				if (e.EditingCanceled)
					SelectedItems.CommitEdit();
			};
		}

		public ObservableCollection<AddressBookRecord> Items { get; set; }

		public ListCollectionView SelectedItems { get; set; }

		private AddressBookRecord _editedRow;
		public AddressBookRecord EditedRow
		{
			get { return _editedRow; }
			set { _editedRow = value; NotifyPropertyChanged("EditedRow"); }
		}

		private bool _isEditing;
		public bool IsEditing
		{
			get { return _isEditing; }
			set { _isEditing = value; NotifyPropertyChanged("IsEditing"); }
		}

		private bool _isAdding;
		public bool IsAdding
		{
			get { return _isAdding; }
			set { _isAdding = value; NotifyPropertyChanged("IsAdding"); }
		}

		private bool SuppressClick;

		public bool AllChecked
		{
			get { return Items.All(i => i.IsChecked); }
			set
			{
				foreach (var curr in Items)
					curr.IsChecked = value;
			}
		}

		private string _searchQuery;
		public string SearchQuery
		{
			get { return _searchQuery; }
			set
			{
				_searchQuery = value;
				NotifyPropertyChanged("SearchQuery");
				SelectedItems.Refresh();
			}
		}

		public event PropertyChangedEventHandler PropertyChanged;
		public void NotifyPropertyChanged(string property)
		{
			var handler = PropertyChanged;
			if (handler != null) handler(this, new PropertyChangedEventArgs(property));
		}


		// ----------------------------------------------
		// Code-Behind part below
		// ----------------------------------------------


		private void XamGrid_CellClicked(object sender, CellClickedEventArgs e)
		{
			Debug.WriteLine("CellClicked");
			if (e.Cell.Control is AddNewRowCellControl || e.Cell.Row.Data.Equals(EditedRow))
				return;

			if (SuppressClick)
			{
				SuppressClick = false;
				// hideNewRow();
				return;
			}

			var record = (AddressBookRecord) e.Cell.Row.Data;
			record.IsChecked = !record.IsChecked;
		}

		private void xamGrid_RowAdded(object sender, RowEventArgs e)
		{
			hideNewRow();

			// Re-position the item to the top and scroll to it
			var added = Items.Last();
			Items.Remove(added);
			Items.Insert(0, added);
			xamGrid.ScrollCellIntoView(xamGrid.Rows[0].Cells[0]);
		}

		private void addContactButton_Click(object sender, RoutedEventArgs e)
		{
			if (IsAdding)
				return;

			if (IsEditing)
			{
				if (EditedRow.IsOkay())
					xamGrid.ExitEditMode(false);
				else
					return;
			}

			SearchQuery = "";
			IsAdding = true;

			showNewRow();
			// Activate the Name editor in the Add New Item row
			var cellCtrl = Infragistics.Windows.Utilities.GetDescendantFromType(xamGrid, typeof(AddNewRowCellControl), true) as AddNewRowCellControl;
			xamGrid.EnterEditMode(cellCtrl.Cell.Row as Row, cellCtrl.Cell.Row.Cells[1]);
		}

		private void confirmAddNewButton_PreviewMouseDown(object sender, MouseButtonEventArgs e)
		{
			xamGrid.ExitEditMode(false);
		}

		private void showNewRow()
		{
			xamGrid.AddNewRowSettings.AllowAddNewRow = AddNewRowLocation.Top;
			xamGrid.UpdateLayout();	
		}

		private void hideNewRow()
		{
			IsAdding = false;
			xamGrid.AddNewRowSettings.AllowAddNewRow = AddNewRowLocation.None;
		}

		private void removeItemCommand_Executed(object sender, ExecutedRoutedEventArgs args)
		{
			var record = args.Parameter as AddressBookRecord;
			Items.Remove(record);
		}

		private void Window_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.Escape && !IsEditing && !IsAdding)
				Close();
		}

		private void xamGrid_PreviewMouseDown(object sender, MouseButtonEventArgs e)
		{
			if (IsEditing || IsAdding)
			{
				var cellsPanel = Infragistics.Windows.Utilities.GetAncestorFromType(e.OriginalSource as DependencyObject, typeof (CellsPanel), false) as CellsPanel;
				if (cellsPanel != null && cellsPanel.DataContext != EditedRow)
					SuppressClick = true;
			}
		}
	}

	public class AddressBookRecord : INotifyPropertyChanged
	{
		private bool _isChecked;
		public bool IsChecked
		{
			get { return _isChecked; }
			set
			{
				_isChecked = value;
				NotifyPropertyChanged("IsChecked");

				var mw = MainWindow.Instance;
				mw.NotifyPropertyChanged("AllChecked");
				mw.NotifyPropertyChanged("DisplayMenu");
			}
		}

		private string _name;
		public string Name
		{
			get { return _name; }
			set
			{
				Debug.WriteLine(string.Format("Name = {0}", value));
				_name = value;
				NotifyPropertyChanged("Name");
			}
		}

		private string _email;
		public string Email
		{
			get { return _email; }
			set
			{
				Debug.WriteLine(string.Format("Email = {0}", value));
				_email = value;
				NotifyPropertyChanged("Email");
			}
		}

		private string _extra;
		public string Extra
		{
			get { return _extra; }
			set
			{
				Debug.WriteLine(string.Format("Extra = {0}", value));
				_extra = value;
				NotifyPropertyChanged("Extra");
			}
		}

		public bool RelatesToQuery(string query)
		{
			if (string.IsNullOrEmpty(query))
				return true;

			var upcaseQuery = query.ToUpperInvariant();
			return (Name != null && Name.ToUpperInvariant().Contains(upcaseQuery)) || (Email != null && Email.ToUpperInvariant().Contains(upcaseQuery));
		}

		public event PropertyChangedEventHandler PropertyChanged;
		protected void NotifyPropertyChanged(string property)
		{
			var handler = PropertyChanged;
			if (handler != null) handler(this, new PropertyChangedEventArgs(property));
		}

		public string ValidateEmail()
		{
			var rx = new Regex(@"^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$", RegexOptions.IgnoreCase);
			return Email == null || rx.IsMatch(Email) ? string.Empty : "The email is invalid!";
		}

		public void RevalidateData()
		{
			NotifyPropertyChanged("Email");
		}
		
		public bool IsOkay()
		{
			return Email != null;
		}
	}
}
